@@ -10,7 +10,7 @@ use {
 // - both fields has a fixed length. Don't use push and pop.
 // - `idxs[0]` contains the number of alive elements
 //   `indx` is positions. So the unused field 0 can hold the last position as a special case.
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, Default)]
 pub struct VarIdHeap {
     /// order : usize -> VarId::from, -- Which var is the n-th best?
     heap: Vec<u32>,
@@ -19,15 +19,6 @@ pub struct VarIdHeap {
     idxs: Vec<u32>,
 }
 
-impl Default for VarIdHeap {
-    fn default() -> VarIdHeap {
-        VarIdHeap {
-            heap: Vec::new(),
-            idxs: Vec::new(),
-        }
-    }
-}
-
 impl fmt::Display for VarIdHeap {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(
