@@ -12,7 +12,7 @@ use {
 const DUMP_INTERVAL: usize = 4096 * 16;
 
 /// Struct for saving UNSAT certification
-#[derive(Debug)]
+#[derive(Debug, Default)]
 pub struct CertificationStore {
     /// clause history to make certification
     queue: Vec<i32>,
@@ -26,16 +26,6 @@ impl Clone for CertificationStore {
     }
 }
 
-impl Default for CertificationStore {
-    fn default() -> Self {
-        CertificationStore {
-            queue: Vec::new(),
-            buffer: None,
-            target: None,
-        }
-    }
-}
-
 impl Instantiate for CertificationStore {
     fn instantiate(config: &Config, _cnf: &CNFDescription) -> Self {
         #[cfg(not(feature = "no_IO"))]
